@@ -23,13 +23,15 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD$");
+__FBSDID("$FreeBSD: src/usr.bin/cpio/test/test_basic.c,v 1.4 2008/08/25 06:39:29 kientzle Exp $");
 
 static void
 verify_files(const char *target)
 {
 	struct stat st, st2;
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	char buff[128];
+#endif
 	int r;
 
 	/*
@@ -42,7 +44,12 @@ verify_files(const char *target)
 	assertEqualInt(r, 0);
 	if (r == 0) {
 		assert(S_ISREG(st.st_mode));
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		/* Group members bits and others bits do not work. */
+		assertEqualInt(0600, st.st_mode & 0700);
+#else
 		assertEqualInt(0644, st.st_mode & 0777);
+#endif
 		assertEqualInt(10, st.st_size);
 		failure("file %s/file should have 2 links", target);
 		assertEqualInt(2, st.st_nlink);
@@ -54,7 +61,12 @@ verify_files(const char *target)
 	assertEqualInt(r, 0);
 	if (r == 0) {
 		assert(S_ISREG(st2.st_mode));
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		/* Group members bits and others bits do not work. */
+		assertEqualInt(0600, st2.st_mode & 0700);
+#else
 		assertEqualInt(0644, st2.st_mode & 0777);
+#endif
 		assertEqualInt(10, st2.st_size);
 		failure("file %s/linkfile should have 2 links", target);
 		assertEqualInt(2, st2.st_nlink);
@@ -69,6 +81,7 @@ verify_files(const char *target)
 	r = lstat("symlink", &st);
 	failure("Failed to stat file %s/symlink, errno=%d", target, errno);
 	assertEqualInt(r, 0);
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	if (r == 0) {
 		failure("symlink should be a symlink; actual mode is %o",
 		    st.st_mode);
@@ -80,13 +93,38 @@ verify_files(const char *target)
 			assertEqualString(buff, "file");
 		}
 	}
+#endif
+
+	/* Another file with 1 link and different permissions. */
+	r = lstat("file2", &st);
+	failure("Failed to stat file %s/file2, errno=%d", target, errno);
+	assertEqualInt(r, 0);
+	if (r == 0) {
+		assert(S_ISREG(st.st_mode));
+		failure("%s/file2: st.st_mode = %o", target, st.st_mode);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		/* Execution bit and group members bits and others
+		 * bits do not work. */
+		assertEqualInt(0600, st.st_mode & 0700);
+#else
+		assertEqualInt(0777, st.st_mode & 0777);
+#endif
+		assertEqualInt(10, st.st_size);
+		failure("file %s/file2 should have 1 link", target);
+		assertEqualInt(1, st.st_nlink);
+	}
 
 	/* dir */
 	r = lstat("dir", &st);
 	if (r == 0) {
 		assertEqualInt(r, 0);
 		assert(S_ISDIR(st.st_mode));
+		failure("%s/dir: st.st_mode = %o", target, st.st_mode);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		assertEqualInt(0700, st.st_mode & 0700);
+#else
 		assertEqualInt(0775, st.st_mode & 0777);
+#endif
 	}
 }
 
@@ -111,7 +149,7 @@ basic_cpio(const char *target,
 
 	/* Verify stderr. */
 	failure("Expected: %s, options=%s", se, pack_options);
-	assertFileContents(se, strlen(se), "pack.err");
+	assertTextFileContents(se, "pack.err");
 
 	/*
 	 * Use cpio to unpack the archive into another directory.
@@ -123,7 +161,7 @@ basic_cpio(const char *target,
 
 	/* Verify stderr. */
 	failure("Error invoking %s -i %s in dir %s", testprog, unpack_options, target);
-	assertFileContents(se, strlen(se), "unpack.err");
+	assertTextFileContents(se, "unpack.err");
 
 	verify_files(target);
 
@@ -141,7 +179,7 @@ passthrough(const char *target)
 	/*
 	 * Use cpio passthrough mode to copy files to another directory.
 	 */
-	r = systemf("%s -p -W quiet %s <filelist >%s/stdout 2>%s/stderr", 
+	r = systemf("%s -p %s <filelist >%s/stdout 2>%s/stderr",
 	    testprog, target, target, target);
 	failure("Error invoking %s -p", testprog);
 	assertEqualInt(r, 0);
@@ -151,7 +189,7 @@ passthrough(const char *target)
 	/* Verify stderr. */
 	failure("Error invoking %s -p in dir %s",
 	    testprog, target);
-	assertEmptyFile("stderr");
+	assertTextFileContents("1 block\n", "stderr");
 
 	verify_files(target);
 	chdir("..");
@@ -185,18 +223,36 @@ DEFINE_TEST(test_basic)
 	assertEqualInt(0, symlink("file", "symlink"));
 	write(filelist, "symlink\n", 8);
 
+	/* Another file with different permissions. */
+	fd = open("file2", O_CREAT | O_WRONLY, 0777);
+	assert(fd >= 0);
+	assertEqualInt(10, write(fd, "123456789", 10));
+	close(fd);
+	write(filelist, "file2\n", 6);
+
 	/* Directory. */
 	assertEqualInt(0, mkdir("dir", 0775));
 	write(filelist, "dir\n", 4);
 	/* All done. */
 	close(filelist);
 
+	umask(022);
+
 	/* Archive/dearchive with a variety of options. */
-	basic_cpio("copy", "", "", "1 block\n");
-	basic_cpio("copy_odc", "--format=odc", "", "1 block\n");
+	basic_cpio("copy", "", "", "2 blocks\n");
+	basic_cpio("copy_odc", "--format=odc", "", "2 blocks\n");
 	basic_cpio("copy_newc", "-H newc", "", "2 blocks\n");
-	basic_cpio("copy_cpio", "-H odc", "", "1 block\n");
-	basic_cpio("copy_ustar", "-H ustar", "", "7 blocks\n");
+	basic_cpio("copy_cpio", "-H odc", "", "2 blocks\n");
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	/*
+	 * On Windows, symbolic link does not work.
+	 * Currentry copying file instead. therefore block size is
+	 * different.
+	 */
+	basic_cpio("copy_ustar", "-H ustar", "", "10 blocks\n");
+#else
+	basic_cpio("copy_ustar", "-H ustar", "", "9 blocks\n");
+#endif
 	/* Copy in one step using -p */
 	passthrough("passthrough");
 
