@@ -23,14 +23,15 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD$");
-
+__FBSDID("$FreeBSD: src/usr.bin/cpio/test/test_gcpio_compat.c,v 1.2 2008/08/22 02:27:06 kientzle Exp $");
 
 static void
 unpack_test(const char *from, const char *options, const char *se)
 {
 	struct stat st, st2;
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	char buff[128];
+#endif
 	int r;
 
 	/* Create a work dir named after the file we're unpacking. */
@@ -48,7 +49,8 @@ unpack_test(const char *from, const char *options, const char *se)
 	assertEqualInt(r, 0);
 
 	/* Verify that nothing went to stderr. */
-	assertFileContents(se, strlen(se), "unpack.err");
+	failure("Error invoking %s -i %s < %s", testprog, options, from);
+	assertTextFileContents(se, "unpack.err");
 
 	/*
 	 * Verify unpacked files.
@@ -60,7 +62,11 @@ unpack_test(const char *from, const char *options, const char *se)
 	assertEqualInt(r, 0);
 	if (r == 0) {
 		assert(S_ISREG(st.st_mode));
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		assertEqualInt(0600, st.st_mode & 0700);
+#else
 		assertEqualInt(0644, st.st_mode & 0777);
+#endif
 		failure("file %s/file", from);
 		assertEqualInt(10, st.st_size);
 		failure("file %s/file", from);
@@ -73,7 +79,11 @@ unpack_test(const char *from, const char *options, const char *se)
 	assertEqualInt(r, 0);
 	if (r == 0) {
 		assert(S_ISREG(st2.st_mode));
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		assertEqualInt(0600, st2.st_mode & 0700);
+#else
 		assertEqualInt(0644, st2.st_mode & 0777);
+#endif
 		failure("file %s/file", from);
 		assertEqualInt(10, st2.st_size);
 		failure("file %s/file", from);
@@ -88,6 +98,7 @@ unpack_test(const char *from, const char *options, const char *se)
 	r = lstat("symlink", &st);
 	failure("Failed to stat file %s/symlink, errno=%d", from, errno);
 	assertEqualInt(r, 0);
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	if (r == 0) {
 		failure("symlink should be a symlink; actual mode is %o",
 		    st.st_mode);
@@ -99,13 +110,18 @@ unpack_test(const char *from, const char *options, const char *se)
 			assertEqualString(buff, "file");
 		}
 	}
+#endif
 
 	/* dir */
 	r = lstat("dir", &st);
 	if (r == 0) {
 		assertEqualInt(r, 0);
 		assert(S_ISDIR(st.st_mode));
+#if defined(_WIN32) && !defined(__CYGWIN__)
+		assertEqualInt(0700, st.st_mode & 0700);
+#else
 		assertEqualInt(0775, st.st_mode & 0777);
+#endif
 	}
 
 	chdir("..");
@@ -121,6 +137,7 @@ DEFINE_TEST(test_gcpio_compat)
 	unpack_test("test_gcpio_compat_ref.bin", "", "1 block\n");
 	unpack_test("test_gcpio_compat_ref.crc", "", "2 blocks\n");
 	unpack_test("test_gcpio_compat_ref.newc", "", "2 blocks\n");
+	/* gcpio-2.9 only reads 6 blocks here */
 	unpack_test("test_gcpio_compat_ref.ustar", "", "7 blocks\n");
 
 	umask(oldumask);
