@@ -24,7 +24,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_cpio.c,v 1.26 2008/01/15 04:56:48 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_cpio.c,v 1.27 2008/12/06 06:45:15 kientzle Exp $");
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -150,7 +150,9 @@ archive_read_support_format_cpio(struct archive *_a)
 
 	r = __archive_read_register_format(a,
 	    cpio,
+	    "cpio",
 	    archive_read_format_cpio_bid,
+	    NULL,
 	    archive_read_format_cpio_read_header,
 	    archive_read_format_cpio_read_data,
 	    NULL,
@@ -165,7 +167,6 @@ archive_read_support_format_cpio(struct archive *_a)
 static int
 archive_read_format_cpio_bid(struct archive_read *a)
 {
-	int bytes_read;
 	const void *h;
 	const unsigned char *p;
 	struct cpio *cpio;
@@ -173,11 +174,7 @@ archive_read_format_cpio_bid(struct archive_read *a)
 
 	cpio = (struct cpio *)(a->format->data);
 
-	bytes_read = (a->decompressor->read_ahead)(a, &h, 6);
-	/* Convert error code into error return. */
-	if (bytes_read < 0)
-		return ((int)bytes_read);
-	if (bytes_read < 6)
+	if ((h = __archive_read_ahead(a, 6, NULL)) == NULL)
 		return (-1);
 
 	p = (const unsigned char *)h;
@@ -228,7 +225,6 @@ archive_read_format_cpio_read_header(struct archive_read *a,
     struct archive_entry *entry)
 {
 	struct cpio *cpio;
-	size_t bytes;
 	const void *h;
 	size_t namelength;
 	size_t name_pad;
@@ -241,21 +237,20 @@ archive_read_format_cpio_read_header(struct archive_read *a,
 		return (r);
 
 	/* Read name from buffer. */
-	bytes = (a->decompressor->read_ahead)(a, &h, namelength + name_pad);
-	if (bytes < namelength + name_pad)
+	h = __archive_read_ahead(a, namelength + name_pad, NULL);
+	if (h == NULL)
 	    return (ARCHIVE_FATAL);
-	(a->decompressor->consume)(a, namelength + name_pad);
+	__archive_read_consume(a, namelength + name_pad);
 	archive_strncpy(&cpio->entry_name, (const char *)h, namelength);
 	archive_entry_set_pathname(entry, cpio->entry_name.s);
 	cpio->entry_offset = 0;
 
 	/* If this is a symlink, read the link contents. */
 	if (archive_entry_filetype(entry) == AE_IFLNK) {
-		bytes = (a->decompressor->read_ahead)(a, &h,
-		    cpio->entry_bytes_remaining);
-		if ((off_t)bytes < cpio->entry_bytes_remaining)
+		h = __archive_read_ahead(a, cpio->entry_bytes_remaining, NULL);
+		if (h == NULL)
 			return (ARCHIVE_FATAL);
-		(a->decompressor->consume)(a, cpio->entry_bytes_remaining);
+		__archive_read_consume(a, cpio->entry_bytes_remaining);
 		archive_strncpy(&cpio->entry_linkname, (const char *)h,
 		    cpio->entry_bytes_remaining);
 		archive_entry_set_symlink(entry, cpio->entry_linkname.s);
@@ -284,7 +279,7 @@ archive_read_format_cpio_read_data(struct archive_read *a,
 
 	cpio = (struct cpio *)(a->format->data);
 	if (cpio->entry_bytes_remaining > 0) {
-		bytes_read = (a->decompressor->read_ahead)(a, buff, 1);
+		*buff = __archive_read_ahead(a, 1, &bytes_read);
 		if (bytes_read <= 0)
 			return (ARCHIVE_FATAL);
 		if (bytes_read > cpio->entry_bytes_remaining)
@@ -293,16 +288,16 @@ archive_read_format_cpio_read_data(struct archive_read *a,
 		*offset = cpio->entry_offset;
 		cpio->entry_offset += bytes_read;
 		cpio->entry_bytes_remaining -= bytes_read;
-		(a->decompressor->consume)(a, bytes_read);
+		__archive_read_consume(a, bytes_read);
 		return (ARCHIVE_OK);
 	} else {
 		while (cpio->entry_padding > 0) {
-			bytes_read = (a->decompressor->read_ahead)(a, buff, 1);
+			*buff = __archive_read_ahead(a, 1, &bytes_read);
 			if (bytes_read <= 0)
 				return (ARCHIVE_FATAL);
 			if (bytes_read > cpio->entry_padding)
 				bytes_read = cpio->entry_padding;
-			(a->decompressor->consume)(a, bytes_read);
+			__archive_read_consume(a, bytes_read);
 			cpio->entry_padding -= bytes_read;
 		}
 		*buff = NULL;
@@ -336,11 +331,12 @@ find_newc_header(struct archive_read *a)
 {
 	const void *h;
 	const char *p, *q;
-	size_t skip, bytes, skipped = 0;
+	size_t skip, skipped = 0;
+	ssize_t bytes;
 
 	for (;;) {
-		bytes = (a->decompressor->read_ahead)(a, &h, 2048);
-		if (bytes < sizeof(struct cpio_newc_header))
+		h = __archive_read_ahead(a, sizeof(struct cpio_newc_header), &bytes);
+		if (h == NULL)
 			return (ARCHIVE_FATAL);
 		p = h;
 		q = p + bytes;
@@ -362,7 +358,7 @@ find_newc_header(struct archive_read *a)
 				if (memcmp("07070", p, 5) == 0
 					&& is_hex(p, sizeof(struct cpio_newc_header))) {
 					skip = p - (const char *)h;
-					(a->decompressor->consume)(a, skip);
+					__archive_read_consume(a, skip);
 					skipped += skip;
 					if (skipped > 0) {
 						archive_set_error(&a->archive,
@@ -385,7 +381,7 @@ find_newc_header(struct archive_read *a)
 			}
 		}
 		skip = p - (const char *)h;
-		(a->decompressor->consume)(a, skip);
+		__archive_read_consume(a, skip);
 		skipped += skip;
 	}
 }
@@ -396,7 +392,6 @@ header_newc(struct archive_read *a, struct cpio *cpio,
 {
 	const void *h;
 	const struct cpio_newc_header *header;
-	size_t bytes;
 	int r;
 
 	r = find_newc_header(a);
@@ -404,10 +399,10 @@ header_newc(struct archive_read *a, struct cpio *cpio,
 		return (r);
 
 	/* Read fixed-size portion of header. */
-	bytes = (a->decompressor->read_ahead)(a, &h, sizeof(struct cpio_newc_header));
-	if (bytes < sizeof(struct cpio_newc_header))
+	h = __archive_read_ahead(a, sizeof(struct cpio_newc_header), NULL);
+	if (h == NULL)
 	    return (ARCHIVE_FATAL);
-	(a->decompressor->consume)(a, sizeof(struct cpio_newc_header));
+	__archive_read_consume(a, sizeof(struct cpio_newc_header));
 
 	/* Parse out hex fields. */
 	header = (const struct cpio_newc_header *)h;
@@ -471,11 +466,12 @@ find_odc_header(struct archive_read *a)
 {
 	const void *h;
 	const char *p, *q;
-	size_t skip, bytes, skipped = 0;
+	size_t skip, skipped = 0;
+	ssize_t bytes;
 
 	for (;;) {
-		bytes = (a->decompressor->read_ahead)(a, &h, 512);
-		if (bytes < sizeof(struct cpio_odc_header))
+		h = __archive_read_ahead(a, sizeof(struct cpio_odc_header), &bytes);
+		if (h == NULL)
 			return (ARCHIVE_FATAL);
 		p = h;
 		q = p + bytes;
@@ -495,7 +491,7 @@ find_odc_header(struct archive_read *a)
 				if (memcmp("070707", p, 6) == 0
 					&& is_octal(p, sizeof(struct cpio_odc_header))) {
 					skip = p - (const char *)h;
-					(a->decompressor->consume)(a, skip);
+					__archive_read_consume(a, skip);
 					skipped += skip;
 					if (skipped > 0) {
 						archive_set_error(&a->archive,
@@ -518,7 +514,7 @@ find_odc_header(struct archive_read *a)
 			}
 		}
 		skip = p - (const char *)h;
-		(a->decompressor->consume)(a, skip);
+		__archive_read_consume(a, skip);
 		skipped += skip;
 	}
 }
@@ -530,7 +526,6 @@ header_odc(struct archive_read *a, struct cpio *cpio,
 	const void *h;
 	int r;
 	const struct cpio_odc_header *header;
-	size_t bytes;
 
 	a->archive.archive_format = ARCHIVE_FORMAT_CPIO_POSIX;
 	a->archive.archive_format_name = "POSIX octet-oriented cpio";
@@ -541,10 +536,10 @@ header_odc(struct archive_read *a, struct cpio *cpio,
 		return (r);
 
 	/* Read fixed-size portion of header. */
-	bytes = (a->decompressor->read_ahead)(a, &h, sizeof(struct cpio_odc_header));
-	if (bytes < sizeof(struct cpio_odc_header))
+	h = __archive_read_ahead(a, sizeof(struct cpio_odc_header), NULL);
+	if (h == NULL)
 	    return (ARCHIVE_FATAL);
-	(a->decompressor->consume)(a, sizeof(struct cpio_odc_header));
+	__archive_read_consume(a, sizeof(struct cpio_odc_header));
 
 	/* Parse out octal fields. */
 	header = (const struct cpio_odc_header *)h;
@@ -578,16 +573,15 @@ header_bin_le(struct archive_read *a, struct cpio *cpio,
 {
 	const void *h;
 	const struct cpio_bin_header *header;
-	size_t bytes;
 
 	a->archive.archive_format = ARCHIVE_FORMAT_CPIO_BIN_LE;
 	a->archive.archive_format_name = "cpio (little-endian binary)";
 
 	/* Read fixed-size portion of header. */
-	bytes = (a->decompressor->read_ahead)(a, &h, sizeof(struct cpio_bin_header));
-	if (bytes < sizeof(struct cpio_bin_header))
+	h = __archive_read_ahead(a, sizeof(struct cpio_bin_header), NULL);
+	if (h == NULL)
 	    return (ARCHIVE_FATAL);
-	(a->decompressor->consume)(a, sizeof(struct cpio_bin_header));
+	__archive_read_consume(a, sizeof(struct cpio_bin_header));
 
 	/* Parse out binary fields. */
 	header = (const struct cpio_bin_header *)h;
@@ -615,17 +609,15 @@ header_bin_be(struct archive_read *a, struct cpio *cpio,
 {
 	const void *h;
 	const struct cpio_bin_header *header;
-	size_t bytes;
 
 	a->archive.archive_format = ARCHIVE_FORMAT_CPIO_BIN_BE;
 	a->archive.archive_format_name = "cpio (big-endian binary)";
 
 	/* Read fixed-size portion of header. */
-	bytes = (a->decompressor->read_ahead)(a, &h,
-	    sizeof(struct cpio_bin_header));
-	if (bytes < sizeof(struct cpio_bin_header))
+	h = __archive_read_ahead(a, sizeof(struct cpio_bin_header), NULL);
+	if (h == NULL)
 	    return (ARCHIVE_FATAL);
-	(a->decompressor->consume)(a, sizeof(struct cpio_bin_header));
+	__archive_read_consume(a, sizeof(struct cpio_bin_header));
 
 	/* Parse out binary fields. */
 	header = (const struct cpio_bin_header *)h;
