@@ -24,7 +24,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_zip.c,v 1.21 2008/02/26 07:17:47 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_zip.c,v 1.28 2008/12/06 06:45:15 kientzle Exp $");
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -36,6 +36,10 @@ __FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_zip.c,v 1.21
 #include <time.h>
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
+#else
+/* Hmmm... This is necessary, but means that we can't correctly extract
+ * even uncompressed entries on platforms that lack zlib. */
+#define	crc32(crc, buf, len) (unsigned long)0
 #endif
 
 #include "archive.h"
@@ -53,6 +57,9 @@ struct zip {
 	int64_t			entry_compressed_bytes_read;
 	int64_t			entry_uncompressed_bytes_read;
 
+	/* Running CRC32 of the decompressed data */
+	unsigned long		entry_crc32;
+
 	unsigned		version;
 	unsigned		system;
 	unsigned		flags;
@@ -68,9 +75,8 @@ struct zip {
 	/* Flags to mark progress of decompression. */
 	char			decompress_init;
 	char			end_of_entry;
-	char			end_of_entry_cleanup;
 
-	long			crc32;
+	unsigned long		crc32;
 	ssize_t			filename_length;
 	ssize_t			extra_length;
 	int64_t			uncompressed_size;
@@ -147,7 +153,9 @@ archive_read_support_format_zip(struct archive *_a)
 
 	r = __archive_read_register_format(a,
 	    zip,
+	    "zip",
 	    archive_read_format_zip_bid,
+	    NULL,
 	    archive_read_format_zip_read_header,
 	    archive_read_format_zip_read_data,
 	    archive_read_format_zip_read_data_skip,
@@ -164,9 +172,9 @@ archive_read_format_zip_bid(struct archive_read *a)
 {
 	const char *p;
 	const void *buff;
-	size_t bytes_avail;
+	ssize_t bytes_avail, offset;
 
-	if ((p = __archive_read_ahead(a, 4)) == NULL)
+	if ((p = __archive_read_ahead(a, 4, NULL)) == NULL)
 		return (-1);
 
 	/*
@@ -186,42 +194,37 @@ archive_read_format_zip_bid(struct archive_read *a)
 	/*
 	 * Attempt to handle self-extracting archives
 	 * by noting a PE header and searching forward
-	 * up to 64k for a 'PK\003\004' marker.
+	 * up to 128k for a 'PK\003\004' marker.
 	 */
 	if (p[0] == 'M' && p[1] == 'Z') {
 		/*
-		 * TODO: Additional checks that this really is a PE
-		 * file before we invoke the 128k lookahead below.
-		 * No point in allocating a bigger lookahead buffer
-		 * if we don't need to.
+		 * TODO: Optimize by initializing 'offset' to an
+		 * estimate of the likely start of the archive data
+		 * based on values in the PE header.  Note that we
+		 * don't need to be exact, but we mustn't skip too
+		 * far.  The search below will compensate if we
+		 * undershoot.
 		 */
-		/*
-		 * TODO: Of course, the compression layer lookahead
-		 * buffers aren't dynamically sized yet; they should be.
-		 */
-		bytes_avail = (a->decompressor->read_ahead)(a, &buff, 128*1024);
-		p = (const char *)buff;
-
-		/*
-		 * TODO: Optimize by jumping forward based on values
-		 * in the PE header.  Note that we don't need to be
-		 * exact, but we mustn't skip too far.  The search
-		 * below will compensate if we undershoot.  Skipping
-		 * will also reduce the chance of false positives
-		 * (which is not really all that high to begin with,
-		 * so maybe skipping isn't really necessary).
-		 */
-
-		while (p < bytes_avail + (const char *)buff) {
-			if (p[0] == 'P' && p[1] == 'K' /* "PK" signature */
-			    && p[2] == 3 && p[3] == 4 /* File entry */
-			    && p[8] == 8 /* compression == deflate */
-			    && p[9] == 0 /* High byte of compression */
-				)
-			{
-				return (30);
+		offset = 0;
+		while (offset < 124000) {
+			/* Get 4k of data beyond where we stopped. */
+			buff = __archive_read_ahead(a, offset + 4096,
+			    &bytes_avail);
+			if (bytes_avail < offset + 1)
+				break;
+			p = (const char *)buff + offset;
+			while (p + 9 < (const char *)buff + bytes_avail) {
+				if (p[0] == 'P' && p[1] == 'K' /* signature */
+				    && p[2] == 3 && p[3] == 4 /* File entry */
+				    && p[8] == 8 /* compression == deflate */
+				    && p[9] == 0 /* High byte of compression */
+					)
+				{
+					return (30);
+				}
+				++p;
 			}
-			++p;
+			offset = p - (const char *)buff;
 		}
 	}
 
@@ -238,7 +241,8 @@ skip_sfx(struct archive_read *a)
 {
 	const void *h;
 	const char *p, *q;
-	size_t skip, bytes;
+	size_t skip;
+	ssize_t bytes;
 
 	/*
 	 * TODO: We should be able to skip forward by a bunch
@@ -248,7 +252,7 @@ skip_sfx(struct archive_read *a)
 	 * reduce the chance of a false positive.
 	 */
 	for (;;) {
-		bytes = (a->decompressor->read_ahead)(a, &h, 4096);
+		h = __archive_read_ahead(a, 4, &bytes);
 		if (bytes < 4)
 			return (ARCHIVE_FATAL);
 		p = h;
@@ -264,7 +268,7 @@ skip_sfx(struct archive_read *a)
 				/* TODO: Additional verification here. */
 				if (memcmp("PK\003\004", p, 4) == 0) {
 					skip = p - (const char *)h;
-					(a->decompressor->consume)(a, skip);
+					__archive_read_consume(a, skip);
 					return (ARCHIVE_OK);
 				}
 				p += 4;
@@ -276,7 +280,7 @@ skip_sfx(struct archive_read *a)
 			}
 		}
 		skip = p - (const char *)h;
-		(a->decompressor->consume)(a, skip);
+		__archive_read_consume(a, skip);
 	}
 }
 
@@ -296,10 +300,10 @@ archive_read_format_zip_read_header(struct archive_read *a,
 	zip = (struct zip *)(a->format->data);
 	zip->decompress_init = 0;
 	zip->end_of_entry = 0;
-	zip->end_of_entry_cleanup = 0;
 	zip->entry_uncompressed_bytes_read = 0;
 	zip->entry_compressed_bytes_read = 0;
-	if ((h = __archive_read_ahead(a, 4)) == NULL)
+	zip->entry_crc32 = crc32(0, NULL, 0);
+	if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
 		return (ARCHIVE_FATAL);
 
 	signature = (const char *)h;
@@ -308,7 +312,7 @@ archive_read_format_zip_read_header(struct archive_read *a,
 		r = skip_sfx(a);
 		if (r < ARCHIVE_WARN)
 			return (r);
-		if ((h = __archive_read_ahead(a, 4)) == NULL)
+		if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
 			return (ARCHIVE_FATAL);
 		signature = (const char *)h;
 	}
@@ -325,8 +329,8 @@ archive_read_format_zip_read_header(struct archive_read *a,
 	 * skip the PK00; the first real file header should follow.
 	 */
 	if (signature[2] == '0' && signature[3] == '0') {
-		(a->decompressor->consume)(a, 4);
-		if ((h = __archive_read_ahead(a, 4)) == NULL)
+		__archive_read_consume(a, 4);
+		if ((h = __archive_read_ahead(a, 4, NULL)) == NULL)
 			return (ARCHIVE_FATAL);
 		signature = (const char *)h;
 		if (signature[0] != 'P' || signature[1] != 'K') {
@@ -370,14 +374,14 @@ archive_read_format_zip_read_header(struct archive_read *a,
 	return (ARCHIVE_FATAL);
 }
 
-int
+static int
 zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
     struct zip *zip)
 {
 	const struct zip_file_header *p;
 	const void *h;
 
-	if ((p = __archive_read_ahead(a, sizeof *p)) == NULL) {
+	if ((p = __archive_read_ahead(a, sizeof *p, NULL)) == NULL) {
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Truncated ZIP file header");
 		return (ARCHIVE_FATAL);
@@ -404,11 +408,11 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 	zip->uncompressed_size = archive_le32dec(p->uncompressed_size);
 	zip->compressed_size = archive_le32dec(p->compressed_size);
 
-	(a->decompressor->consume)(a, sizeof(struct zip_file_header));
+	__archive_read_consume(a, sizeof(struct zip_file_header));
 
 
 	/* Read the filename. */
-	if ((h = __archive_read_ahead(a, zip->filename_length)) == NULL) {
+	if ((h = __archive_read_ahead(a, zip->filename_length, NULL)) == NULL) {
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Truncated ZIP file header");
 		return (ARCHIVE_FATAL);
@@ -416,7 +420,7 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 	if (archive_string_ensure(&zip->pathname, zip->filename_length) == NULL)
 		__archive_errx(1, "Out of memory");
 	archive_strncpy(&zip->pathname, h, zip->filename_length);
-	(a->decompressor->consume)(a, zip->filename_length);
+	__archive_read_consume(a, zip->filename_length);
 	archive_entry_set_pathname(entry, zip->pathname.s);
 
 	if (zip->pathname.s[archive_strlen(&zip->pathname) - 1] == '/')
@@ -425,13 +429,13 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 		zip->mode = AE_IFREG | 0777;
 
 	/* Read the extra data. */
-	if ((h = __archive_read_ahead(a, zip->extra_length)) == NULL) {
+	if ((h = __archive_read_ahead(a, zip->extra_length, NULL)) == NULL) {
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Truncated ZIP file header");
 		return (ARCHIVE_FATAL);
 	}
 	process_extra(h, zip);
-	(a->decompressor->consume)(a, zip->extra_length);
+	__archive_read_consume(a, zip->extra_length);
 
 	/* Populate some additional entry fields: */
 	archive_entry_set_mode(entry, zip->mode);
@@ -440,7 +444,9 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 	archive_entry_set_mtime(entry, zip->mtime, 0);
 	archive_entry_set_ctime(entry, zip->ctime, 0);
 	archive_entry_set_atime(entry, zip->atime, 0);
-	archive_entry_set_size(entry, zip->uncompressed_size);
+	/* Set the size only if it's meaningful. */
+	if (0 == (zip->flags & ZIP_LENGTH_AT_END))
+		archive_entry_set_size(entry, zip->uncompressed_size);
 
 	zip->entry_bytes_remaining = zip->compressed_size;
 	zip->entry_offset = 0;
@@ -494,46 +500,6 @@ archive_read_format_zip_read_data(struct archive_read *a,
 	 * ARCHIVE_EOF this time.
 	 */
 	if (zip->end_of_entry) {
-		if (!zip->end_of_entry_cleanup) {
-			if (zip->flags & ZIP_LENGTH_AT_END) {
-				const char *p;
-
-				if ((p = __archive_read_ahead(a, 16)) == NULL) {
-					archive_set_error(&a->archive,
-					    ARCHIVE_ERRNO_FILE_FORMAT,
-					    "Truncated ZIP end-of-file record");
-					return (ARCHIVE_FATAL);
-				}
-				zip->crc32 = archive_le32dec(p + 4);
-				zip->compressed_size = archive_le32dec(p + 8);
-				zip->uncompressed_size = archive_le32dec(p + 12);
-				(a->decompressor->consume)(a, 16);
-			}
-
-			/* Check file size, CRC against these values. */
-			if (zip->compressed_size != zip->entry_compressed_bytes_read) {
-				archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-				    "ZIP compressed data is wrong size");
-				return (ARCHIVE_WARN);
-			}
-			/* Size field only stores the lower 32 bits of the actual size. */
-			if ((zip->uncompressed_size & UINT32_MAX)
-			    != (zip->entry_uncompressed_bytes_read & UINT32_MAX)) {
-				archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-				    "ZIP uncompressed data is wrong size");
-				return (ARCHIVE_WARN);
-			}
-/* TODO: Compute CRC. */
-/*
-			if (zip->crc32 != zip->entry_crc32_calculated) {
-				archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-				    "ZIP data CRC error");
-				return (ARCHIVE_WARN);
-			}
-*/
-			/* End-of-entry cleanup done. */
-			zip->end_of_entry_cleanup = 1;
-		}
 		*offset = zip->entry_uncompressed_bytes_read;
 		*size = 0;
 		*buff = NULL;
@@ -564,13 +530,60 @@ archive_read_format_zip_read_data(struct archive_read *a,
 			 */
 			r = ARCHIVE_FATAL;
 		} else {
-			/* We know compressed size; just skip it. */
-			archive_read_format_zip_read_data_skip(a);
+			/* We can't decompress this entry, but we will
+			 * be able to skip() it and try the next entry. */
 			r = ARCHIVE_WARN;
 		}
 		break;
 	}
-	return (r);
+	if (r != ARCHIVE_OK)
+		return (r);
+	/* Update checksum */
+	if (*size)
+		zip->entry_crc32 =
+		    crc32(zip->entry_crc32, *buff, *size);
+	/* If we hit the end, swallow any end-of-data marker. */
+	if (zip->end_of_entry) {
+		if (zip->flags & ZIP_LENGTH_AT_END) {
+			const char *p;
+
+			if ((p = __archive_read_ahead(a, 16, NULL)) == NULL) {
+				archive_set_error(&a->archive,
+				    ARCHIVE_ERRNO_FILE_FORMAT,
+				    "Truncated ZIP end-of-file record");
+				return (ARCHIVE_FATAL);
+			}
+			zip->crc32 = archive_le32dec(p + 4);
+			zip->compressed_size = archive_le32dec(p + 8);
+			zip->uncompressed_size = archive_le32dec(p + 12);
+			__archive_read_consume(a, 16);
+		}
+		/* Check file size, CRC against these values. */
+		if (zip->compressed_size != zip->entry_compressed_bytes_read) {
+			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+			    "ZIP compressed data is wrong size");
+			return (ARCHIVE_WARN);
+		}
+		/* Size field only stores the lower 32 bits of the actual size. */
+		if ((zip->uncompressed_size & UINT32_MAX)
+		    != (zip->entry_uncompressed_bytes_read & UINT32_MAX)) {
+			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+			    "ZIP uncompressed data is wrong size");
+			return (ARCHIVE_WARN);
+		}
+		/* Check computed CRC against header */
+		if (zip->crc32 != zip->entry_crc32) {
+			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+			    "ZIP bad CRC: 0x%lx should be 0x%lx",
+			    zip->entry_crc32, zip->crc32);
+			return (ARCHIVE_WARN);
+		}
+	}
+
+	/* Return EOF immediately if this is a non-regular file. */
+	if (AE_IFREG != (zip->mode & AE_IFMT))
+		return (ARCHIVE_EOF);
+	return (ARCHIVE_OK);
 }
 
 /*
@@ -608,7 +621,7 @@ zip_read_data_none(struct archive_read *a, const void **buff,
 	 * available bytes; asking for more than that forces the
 	 * decompressor to combine reads by copying data.
 	 */
-	bytes_avail = (a->decompressor->read_ahead)(a, buff, 1);
+	*buff = __archive_read_ahead(a, 1, &bytes_avail);
 	if (bytes_avail <= 0) {
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Truncated ZIP file data");
@@ -616,7 +629,7 @@ zip_read_data_none(struct archive_read *a, const void **buff,
 	}
 	if (bytes_avail > zip->entry_bytes_remaining)
 		bytes_avail = zip->entry_bytes_remaining;
-	(a->decompressor->consume)(a, bytes_avail);
+	__archive_read_consume(a, bytes_avail);
 	*size = bytes_avail;
 	*offset = zip->entry_offset;
 	zip->entry_offset += *size;
@@ -674,7 +687,7 @@ zip_read_data_deflate(struct archive_read *a, const void **buff,
 	 * available bytes; asking for more than that forces the
 	 * decompressor to combine reads by copying data.
 	 */
-	bytes_avail = (a->decompressor->read_ahead)(a, &compressed_buff, 1);
+	compressed_buff = __archive_read_ahead(a, 1, &bytes_avail);
 	if (bytes_avail <= 0) {
 		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Truncated ZIP file body");
@@ -713,7 +726,7 @@ zip_read_data_deflate(struct archive_read *a, const void **buff,
 
 	/* Consume as much as the compressor actually used. */
 	bytes_avail = zip->stream.total_in;
-	(a->decompressor->consume)(a, bytes_avail);
+	__archive_read_consume(a, bytes_avail);
 	zip->entry_bytes_remaining -= bytes_avail;
 	zip->entry_compressed_bytes_read += bytes_avail;
 
@@ -743,10 +756,14 @@ archive_read_format_zip_read_data_skip(struct archive_read *a)
 {
 	struct zip *zip;
 	const void *buff = NULL;
-	ssize_t bytes_avail;
+	off_t bytes_skipped;
 
 	zip = (struct zip *)(a->format->data);
 
+	/* If we've already read to end of data, we're done. */
+	if (zip->end_of_entry)
+		return (ARCHIVE_OK);
+
 	/*
 	 * If the length is at the end, we have no choice but
 	 * to decompress all the data to find the end marker.
@@ -766,21 +783,12 @@ archive_read_format_zip_read_data_skip(struct archive_read *a)
 	 * If the length is at the beginning, we can skip the
 	 * compressed data much more quickly.
 	 */
-	while (zip->entry_bytes_remaining > 0) {
-		bytes_avail = (a->decompressor->read_ahead)(a, &buff, 1);
-		if (bytes_avail <= 0) {
-			archive_set_error(&a->archive,
-			    ARCHIVE_ERRNO_FILE_FORMAT,
-			    "Truncated ZIP file body");
-			return (ARCHIVE_FATAL);
-		}
-		if (bytes_avail > zip->entry_bytes_remaining)
-			bytes_avail = zip->entry_bytes_remaining;
-		(a->decompressor->consume)(a, bytes_avail);
-		zip->entry_bytes_remaining -= bytes_avail;
-	}
+	bytes_skipped = __archive_read_skip(a, zip->entry_bytes_remaining);
+	if (bytes_skipped < 0)
+		return (ARCHIVE_FATAL);
+
 	/* This entry is finished and done. */
-	zip->end_of_entry_cleanup = zip->end_of_entry = 1;
+	zip->end_of_entry = 1;
 	return (ARCHIVE_OK);
 }
 
