@@ -7,23 +7,32 @@ DEFINE_TEST(test_compat_bzip2)
 DEFINE_TEST(test_compat_gtar)
 DEFINE_TEST(test_compat_gzip)
 DEFINE_TEST(test_compat_tar_hardlink)
+DEFINE_TEST(test_compat_xz)
 DEFINE_TEST(test_compat_zip)
 DEFINE_TEST(test_empty_write)
 DEFINE_TEST(test_entry)
 DEFINE_TEST(test_entry_strmode)
+DEFINE_TEST(test_extattr_freebsd)
 DEFINE_TEST(test_fuzz)
 DEFINE_TEST(test_link_resolver)
+DEFINE_TEST(test_open_fd)
+DEFINE_TEST(test_open_file)
+DEFINE_TEST(test_open_filename)
 DEFINE_TEST(test_pax_filename_encoding_1)
 DEFINE_TEST(test_pax_filename_encoding_2)
 DEFINE_TEST(test_pax_filename_encoding_3)
 DEFINE_TEST(test_read_compress_program)
 DEFINE_TEST(test_read_data_large)
+DEFINE_TEST(test_read_disk)
+DEFINE_TEST(test_read_disk_entry_from_file)
 DEFINE_TEST(test_read_extract)
+DEFINE_TEST(test_read_file_nonexistent)
 DEFINE_TEST(test_read_format_ar)
 DEFINE_TEST(test_read_format_cpio_bin)
 DEFINE_TEST(test_read_format_cpio_bin_Z)
 DEFINE_TEST(test_read_format_cpio_bin_bz2)
 DEFINE_TEST(test_read_format_cpio_bin_gz)
+DEFINE_TEST(test_read_format_cpio_bin_xz)
 DEFINE_TEST(test_read_format_cpio_odc)
 DEFINE_TEST(test_read_format_cpio_svr4_gzip)
 DEFINE_TEST(test_read_format_cpio_svr4c_Z)
@@ -32,6 +41,8 @@ DEFINE_TEST(test_read_format_gtar_gz)
 DEFINE_TEST(test_read_format_gtar_lzma)
 DEFINE_TEST(test_read_format_gtar_sparse)
 DEFINE_TEST(test_read_format_iso_gz)
+DEFINE_TEST(test_read_format_isojoliet_bz2)
+DEFINE_TEST(test_read_format_isojolietrr_bz2)
 DEFINE_TEST(test_read_format_isorr_bz2)
 DEFINE_TEST(test_read_format_mtree)
 DEFINE_TEST(test_read_format_pax_bz2)
@@ -39,6 +50,7 @@ DEFINE_TEST(test_read_format_tar)
 DEFINE_TEST(test_read_format_tar_empty_filename)
 DEFINE_TEST(test_read_format_tbz)
 DEFINE_TEST(test_read_format_tgz)
+DEFINE_TEST(test_read_format_txz)
 DEFINE_TEST(test_read_format_tz)
 DEFINE_TEST(test_read_format_zip)
 DEFINE_TEST(test_read_large)
@@ -49,17 +61,24 @@ DEFINE_TEST(test_tar_filenames)
 DEFINE_TEST(test_tar_large)
 DEFINE_TEST(test_ustar_filenames)
 DEFINE_TEST(test_write_compress)
+DEFINE_TEST(test_write_compress_bzip2)
+DEFINE_TEST(test_write_compress_gzip)
+DEFINE_TEST(test_write_compress_lzma)
 DEFINE_TEST(test_write_compress_program)
+DEFINE_TEST(test_write_compress_xz)
 DEFINE_TEST(test_write_disk)
+DEFINE_TEST(test_write_disk_failures)
 DEFINE_TEST(test_write_disk_hardlink)
 DEFINE_TEST(test_write_disk_perms)
 DEFINE_TEST(test_write_disk_secure)
+DEFINE_TEST(test_write_disk_sparse)
 DEFINE_TEST(test_write_disk_times)
 DEFINE_TEST(test_write_format_ar)
 DEFINE_TEST(test_write_format_cpio)
 DEFINE_TEST(test_write_format_cpio_empty)
 DEFINE_TEST(test_write_format_cpio_newc)
 DEFINE_TEST(test_write_format_cpio_odc)
+DEFINE_TEST(test_write_format_mtree)
 DEFINE_TEST(test_write_format_pax)
 DEFINE_TEST(test_write_format_shar_empty)
 DEFINE_TEST(test_write_format_tar)
