@@ -23,7 +23,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_acl_pax.c,v 1.4 2007/07/06 15:43:11 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/test/test_acl_pax.c,v 1.6 2008/09/01 05:38:33 kientzle Exp $");
 
 /*
  * Exercise the system-independent portion of the ACL support.
@@ -332,14 +332,10 @@ acl_match(struct acl_t *acl, int type, int permset, int tag, int qual, const cha
 		return (1);
 	if (qual != acl->qual)
 		return (0);
-	if (name == NULL) {
-		if (acl->name == NULL || acl->name[0] == '\0')
-			return (1);
-	}
-	if (acl->name == NULL) {
-		if (name[0] == '\0')
-			return (1);
-	}
+	if (name == NULL)
+		return (acl->name == NULL || acl->name[0] == '\0');
+	if (acl->name == NULL)
+		return (name == NULL || name[0] == '\0');
 	return (0 == strcmp(name, acl->name));
 }
 
@@ -390,7 +386,7 @@ compare_acls(struct archive_entry *ae, struct acl_t *acls, int n, int mode)
 			assert(matched == 1);
 		}
 	}
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 	/* Known broken before 1.9.0. */
 	skipping("archive_entry_acl_next() exits with ARCHIVE_EOF");
 #else
@@ -450,15 +446,15 @@ DEFINE_TEST(test_acl_pax)
 
 	/* Close out the archive. */
 	assertA(0 == archive_write_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assertA(0 == archive_write_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_write_finish(a);
+#else
+	assertA(0 == archive_write_finish(a));
 #endif
 
 	/* Write out the data we generated to a file for manual inspection. */
 	assert(-1 < (fd = open("testout", O_WRONLY | O_CREAT | O_TRUNC, 0775)));
-	assert(used == (size_t)write(fd, buff, used));
+	assert(used == (size_t)write(fd, buff, (unsigned int)used));
 	close(fd);
 
 	/* Write out the reference data to a file for manual inspection. */
@@ -470,7 +466,7 @@ DEFINE_TEST(test_acl_pax)
 	failure("Generated pax archive does not match reference; check 'testout' and 'reference' files.");
 	assert(0 == memcmp(buff, reference, sizeof(reference)));
 	failure("Generated pax archive does not match reference; check 'testout' and 'reference' files.");
-	assertEqualInt(used, sizeof(reference));
+	assertEqualInt((int)used, sizeof(reference));
 
 	/* Read back each entry and check that the ACL data is right. */
 	assert(NULL != (a = archive_read_new()));
@@ -513,9 +509,9 @@ DEFINE_TEST(test_acl_pax)
 
 	/* Close the archive. */
 	assertA(0 == archive_read_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_read_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_read_finish(a);
+#else
+	assertA(0 == archive_read_finish(a));
 #endif
 }
