@@ -23,7 +23,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_read_extract.c,v 1.4 2008/06/15 10:35:22 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/test/test_read_extract.c,v 1.5 2008/09/01 05:38:33 kientzle Exp $");
 
 #define BUFF_SIZE 1000000
 #define FILE_BUFF_SIZE 100000
@@ -32,12 +32,16 @@ DEFINE_TEST(test_read_extract)
 {
 	struct archive_entry *ae;
 	struct archive *a;
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	struct stat st;
+#endif
 	size_t used;
 	int i;
 	char *buff, *file_buff;
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	int fd;
 	ssize_t bytes_read;
+#endif
 
 	buff = malloc(BUFF_SIZE);
 	file_buff = malloc(FILE_BUFF_SIZE);
@@ -105,10 +109,10 @@ DEFINE_TEST(test_read_extract)
 	archive_entry_free(ae);
 	/* Close out the archive. */
 	assertA(0 == archive_write_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assertA(0 == archive_write_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_write_finish(a);
+#else
+	assertA(0 == archive_write_finish(a));
 #endif
 
 	/* Extract the entries to disk. */
@@ -128,12 +132,13 @@ DEFINE_TEST(test_read_extract)
 	}
 	assertA(ARCHIVE_EOF == archive_read_next_header(a, &ae));
 	assert(0 == archive_read_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_read_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_read_finish(a);
+#else
+	assert(0 == archive_read_finish(a));
 #endif
 
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	/* Test the entries on disk. */
 	/* This first entry was extracted with ARCHIVE_EXTRACT_PERM,
 	 * so the permissions should have been restored exactly,
@@ -153,6 +158,7 @@ DEFINE_TEST(test_read_extract)
 	failure("The file on disk could not be opened.");
 	assert(fd != 0);
 	bytes_read = read(fd, buff, FILE_BUFF_SIZE);
+	close(fd);
 	failure("The file contents read from disk are the wrong size");
 	assert(bytes_read == FILE_BUFF_SIZE);
 	failure("The file contents on disk do not match the file contents that were put into the archive.");
@@ -187,6 +193,7 @@ DEFINE_TEST(test_read_extract)
 #endif
 	assert(0 == stat("symlink", &st));
 	assert(st.st_mode == (S_IFREG | 0755));
+#endif
 
 	free(buff);
 	free(file_buff);
