@@ -36,6 +36,7 @@ DEFINE_TEST(test_read_format_cpio_bin_bz2)
 {
 	struct archive_entry *ae;
 	struct archive *a;
+
 	assert((a = archive_read_new()) != NULL);
 	assertEqualIntA(a, ARCHIVE_OK,
 	    archive_read_support_compression_all(a));
@@ -46,11 +47,7 @@ DEFINE_TEST(test_read_format_cpio_bin_bz2)
 	assert(archive_compression(a) == ARCHIVE_COMPRESSION_BZIP2);
 	assert(archive_format(a) == ARCHIVE_FORMAT_CPIO_BIN_LE);
 	assert(0 == archive_read_close(a));
-#if ARCHIVE_VERSION_NUMBER < 2000000
-	archive_read_finish(a);
-#else
-	assert(0 == archive_read_finish(a));
-#endif
+	assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
 }
 
 
