@@ -23,7 +23,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_format_cpio_newc.c,v 1.2 2008/01/23 05:43:26 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_format_cpio_newc.c,v 1.3 2008/09/01 05:38:33 kientzle Exp $");
 
 
 static int
@@ -111,10 +111,10 @@ DEFINE_TEST(test_write_format_cpio_newc)
 	archive_entry_free(entry);
 
 
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_write_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_write_finish(a);
+#else
+	assert(0 == archive_write_finish(a));
 #endif
 
 	/*
@@ -205,7 +205,7 @@ DEFINE_TEST(test_write_format_cpio_newc)
 	assertEqualMem(e + 121, "\0\0\0", 3); /* Pad to multiple of 4 bytes */
 	e += 124; /* Must be multiple of four here! */
 
-	assertEqualInt(used, e - buff);
+	assertEqualInt((int)used, e - buff);
 
 	free(buff);
 }
