@@ -24,7 +24,7 @@
  */
 
 #include "bsdtar_platform.h"
-__FBSDID("$FreeBSD: src/usr.bin/tar/read.c,v 1.35 2008/01/02 00:21:27 kientzle Exp $");
+__FBSDID("$FreeBSD: src/usr.bin/tar/read.c,v 1.40 2008/08/21 06:41:14 kientzle Exp $");
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -77,12 +77,28 @@ void
 tar_mode_t(struct bsdtar *bsdtar)
 {
 	read_archive(bsdtar, 't');
+	unmatched_inclusions_warn(bsdtar, "Not found in archive");
 }
 
 void
 tar_mode_x(struct bsdtar *bsdtar)
 {
+	/* We want to catch SIGINFO and SIGUSR1. */
+	siginfo_init(bsdtar);
+
 	read_archive(bsdtar, 'x');
+
+	unmatched_inclusions_warn(bsdtar, "Not found in archive");
+	/* Restore old SIGINFO + SIGUSR1 handlers. */
+	siginfo_done(bsdtar);
+}
+
+static void
+progress_func(void * cookie)
+{
+	struct bsdtar * bsdtar = cookie;
+
+	siginfo_printinfo(bsdtar, 0);
 }
 
 /*
@@ -111,6 +127,8 @@ read_archive(struct bsdtar *bsdtar, char mode)
 	else
 		archive_read_support_compression_all(a);
 	archive_read_support_format_all(a);
+	if (ARCHIVE_OK != archive_read_set_options(a, bsdtar->option_options))
+		bsdtar_errc(bsdtar, 1, 0, archive_error_string(a));
 	if (archive_read_open_file(a, bsdtar->filename,
 	    bsdtar->bytes_per_block != 0 ? bsdtar->bytes_per_block :
 	    DEFAULT_BYTES_PER_BLOCK))
@@ -118,6 +136,23 @@ read_archive(struct bsdtar *bsdtar, char mode)
 		    archive_error_string(a));
 
 	do_chdir(bsdtar);
+
+	if (mode == 'x') {
+		/* Set an extract callback so that we can handle SIGINFO. */
+		archive_read_extract_set_progress_callback(a, progress_func,
+		    bsdtar);
+	}
+
+	if (mode == 'x' && bsdtar->option_chroot) {
+#if HAVE_CHROOT
+		if (chroot(".") != 0)
+			bsdtar_errc(bsdtar, 1, errno, "Can't chroot to \".\"");
+#else
+		bsdtar_errc(bsdtar, 1, 0,
+		    "chroot isn't supported on this platform");
+#endif
+	}
+
 	for (;;) {
 		/* Support --fast-read option */
 		if (bsdtar->option_fast_read &&
@@ -139,6 +174,11 @@ read_archive(struct bsdtar *bsdtar, char mode)
 		if (r == ARCHIVE_FATAL)
 			break;
 
+		if (bsdtar->option_numeric_owner) {
+			archive_entry_set_uname(entry, NULL);
+			archive_entry_set_gname(entry, NULL);
+		}
+
 		/*
 		 * Exclude entries that are too old.
 		 */
@@ -172,22 +212,17 @@ read_archive(struct bsdtar *bsdtar, char mode)
 		if (excluded(bsdtar, archive_entry_pathname(entry)))
 			continue; /* Excluded by a pattern test. */
 
-		/*
-		 * Modify the pathname as requested by the user.  We
-		 * do this for -t as well to give users a way to
-		 * preview the effects of their rewrites.  We also do
-		 * this before extraction security checks (including
-		 * leading '/' removal).  Note that some rewrite
-		 * failures prevent extraction.
-		 */
-		if (edit_pathname(bsdtar, entry))
-			continue; /* Excluded by a rewrite failure. */
-
 		if (mode == 't') {
 			/* Perversely, gtar uses -O to mean "send to stderr"
 			 * when used with -t. */
 			out = bsdtar->option_stdout ? stderr : stdout;
 
+			/*
+			 * TODO: Provide some reasonable way to
+			 * preview rewrites.  gtar always displays
+			 * the unedited path in -t output, which means
+			 * you cannot easily preview rewrites.
+			 */
 			if (bsdtar->verbose < 2)
 				safe_fprintf(out, "%s",
 				    archive_entry_pathname(entry));
@@ -214,6 +249,10 @@ read_archive(struct bsdtar *bsdtar, char mode)
 			}
 			fprintf(out, "\n");
 		} else {
+			/* Note: some rewrite failures prevent extraction. */
+			if (edit_pathname(bsdtar, entry))
+				continue; /* Excluded by a rewrite failure. */
+
 			if (bsdtar->option_interactive &&
 			    !yes("extract '%s'", archive_entry_pathname(entry)))
 				continue;
@@ -227,6 +266,12 @@ read_archive(struct bsdtar *bsdtar, char mode)
 				    archive_entry_pathname(entry));
 				fflush(stderr);
 			}
+
+			/* Tell the SIGINFO-handler code what we're doing. */
+			siginfo_setinfo(bsdtar, "extracting",
+			    archive_entry_pathname(entry), 0);
+			siginfo_printinfo(bsdtar, 0);
+
 			if (bsdtar->option_stdout)
 				r = archive_read_data_into_fd(a, 1);
 			else
@@ -249,6 +294,13 @@ read_archive(struct bsdtar *bsdtar, char mode)
 		}
 	}
 
+
+	r = archive_read_close(a);
+	if (r != ARCHIVE_OK)
+		bsdtar_warnc(bsdtar, 0, "%s", archive_error_string(a));
+	if (r <= ARCHIVE_WARN)
+		bsdtar->return_value = 1;
+
 	if (bsdtar->verbose > 2)
 		fprintf(stdout, "Archive Format: %s,  Compression: %s\n",
 		    archive_format_name(a), archive_compression_name(a));
@@ -341,10 +393,18 @@ list_item_verbose(struct bsdtar *bsdtar, FILE *out, struct archive_entry *entry)
 
 	/* Format the time using 'ls -l' conventions. */
 	tim = (time_t)st->st_mtime;
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	/* Windows' strftime function does not support %e format. */
+	if (abs(tim - now) > (365/2)*86400)
+		fmt = bsdtar->day_first ? "%d %b  %Y" : "%b %d  %Y";
+	else
+		fmt = bsdtar->day_first ? "%d %b %H:%M" : "%b %d %H:%M";
+#else
 	if (abs(tim - now) > (365/2)*86400)
 		fmt = bsdtar->day_first ? "%e %b  %Y" : "%b %e  %Y";
 	else
-		fmt = bsdtar->day_first ? "%e %b %R" : "%b %e %R";
+		fmt = bsdtar->day_first ? "%e %b %H:%M" : "%b %e %H:%M";
+#endif
 	strftime(tmp, sizeof(tmp), fmt, localtime(&tim));
 	fprintf(out, " %s ", tmp);
 	safe_fprintf(out, "%s", archive_entry_pathname(entry));
